Black = 'BLACK'
Blue = 'BLUE'
Brown = 'BROWN'
Clear = 'CLEAR'
Cyan = 'CYAN'
DarkGray = 'DARKGRAY'
Gray = 'GRAY'
Green = 'GREEN'
LightGray = 'LIGHTGRAY'
Magenta = 'MAGENTA'
Orange = 'ORANGE'
Purple = 'PURPLE'
Red = 'RED'
White = 'WHITE'
Yellow = 'YELLOW'
